#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

select_boot_setup() {
    # choose bootloader and by extension mountpoint (if needed)
    tput civis
    BOOTLOADER="$(dialog --cr-wrap --stdout --backtitle "$BT" \
        --title " $_PrepMount " --menu "$_MntBootBody" 0 0 0 ${BOOTLOADERS[$SYS]})"
    [[ $? != 0 || $BOOTLOADER == "" ]] && return 1

    if [[ $SYS == 'BIOS' && $BOOTLOADER == 'grub' && $BOOT_DEVICE == "" ]]; then
        # grub on BIOS needs an install device, NOT partition eg. /dev/sda
        select_device 'boot' || return 1
    fi

    if [[ $BOOTLOADER == 'systemd-boot' ]]; then
        EDIT_FILES[9]="/boot/loader/entries/$DIST.conf"
    elif [[ $BOOTLOADER == 'syslinux' && $SYS == 'BIOS' ]]; then
        BOOT_CMDS[$BOOTLOADER]="$(dialog --cr-wrap --stdout --backtitle "$BT" \
            --title " $_InstSysTitle " --menu "$_InstSysBody" 0 0 0 \
            "syslinux-install_update -iam" "Install to MBR (Master Boot Record)" \
            "syslinux-install_update -i" "Install to root partition (/)")"
        [[ $? != 0 || ${BOOT_CMDS[$BOOTLOADER]} == "" ]] && return 1
    else
        EDIT_FILES[9]="/etc/default/grub"
    fi

    return 0
}

shim_secure_boot() {
    # still a W.I.P

    local shim_file="shim64.efi"
    [[ $IS_64BIT != true ]] && shim_file="shim.efi"
    efibootmgr -c -w -L $DIST -d $BOOT_DEVICE -p $BOOT_PART_NUM -l ${MNT}${BOOT_MNTS[$SYS-$BOOTLOADER]}/$shim_file
    return 0
}

uefi_boot_fallback() {
    # some UEFI firmware is finicky and requires a specific folder in
    # /boot/efi/EFI/ and named 'boot', 'Boot', or 'BOOT'
    # copy the bootloaders efi stub to that directory as bootx64.efi

    local default="boot"
    local esp="${MNT}${BOOT_MNTS[$SYS-$BOOTLOADER]}"
    for i in $(find "$esp/EFI/" -maxdepth 1 -mindepth 1 -type d 2>/dev/null); do
        grep -qi "boot" <<< "$(basename $i)" && { default="$(basename $i)"; break; }
    done

    if [[ -d $esp/EFI/$default ]]; then
        rm -f $esp/EFI/$default/*
    else
        mkdir -p $esp/EFI/$default
    fi

    if [[ $1 == 'syslinux' ]]; then
        cp -rf $esp/EFI/$1/* $esp/EFI/$default/
        cp -f $esp/EFI/$1/syslinux.efi $esp/EFI/$default/bootx64.efi
    else
        local grub_file="grubx64.efi"
        local boot_file="bootx64.efi"
        if [[ $IS_64BIT != true ]]; then
            local grub_file="grubia32.efi"
            local boot_file="bootia32.efi"
        fi
        cp -f $esp/EFI/$1/$grub_file $esp/EFI/$default/$boot_file
    fi
    return 0
}

prep_for_grub() {
    local cfg="$MNT/etc/default/grub"

    if [[ $SYS == 'UEFI' ]]; then
        local gtype='x86_64-efi'
        [[ $IS_64BIT != true ]] && gtype='i386-efi'
        BOOT_CMDS[grub]="${BOOT_CMDS[grub]} --target=$gtype --bootloader-id=$DIST"
    else
        BOOT_CMDS[grub]="${BOOT_CMDS[grub]} --target=i386-pc $BOOT_DEVICE"
    fi

    BOOT_CMDS[grub]="${BOOT_CMDS[grub]} && grub-mkconfig -o /boot/grub/grub.cfg"

    sed -i "s/GRUB_DISTRIBUTOR=.*/GRUB_DISTRIBUTOR=\"${DIST}\"/g;
            s/GRUB_CMDLINE_LINUX_DEFAULT=.*/GRUB_CMDLINE_LINUX_DEFAULT=\"\"/g" $cfg

    if (( LUKS == 1 )); then
        sed -i "s~#GRUB_ENABLE_CRYPTODISK~GRUB_ENABLE_CRYPTODISK~g;
                s~GRUB_CMDLINE_LINUX=.*~GRUB_CMDLINE_LINUX=\"${LUKS_DEV}\"~g" $cfg
    fi

    if [[ $SYS != 'UEFI' && $LVM -eq 1 && $SEPERATE_BOOT -eq 0 ]]; then
        sed -i "s/GRUB_PRELOAD_MODULES=.*/GRUB_PRELOAD_MODULES=\"lvm\"/g" $cfg
    fi

    return 0
}

prep_for_systemd-boot() {
    # create the boot entry configs
    mkdir -p $MNT/boot/loader/entries
    cat > $MNT/boot/loader/loader.conf << EOF
default  $DIST
timeout  5
editor   no
EOF
cat > $MNT/boot/loader/entries/${DIST}.conf << EOF
title   $DIST Linux
linux   /vmlinuz-${KERNEL}$([[ $(grep 'GenuineIntel' /proc/cpuinfo) && -e $MNT/boot/intel-ucode.img ]] && echo -en "\ninitrd  /intel-ucode.img")
initrd  /initramfs-$KERNEL.img
options root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV")rw
EOF

    # add pacman hook to update the bootloader when systemd receives an update
    mkdir -p $MNT/pacman.d/hooks
    cat > $MNT/pacman.d/hooks/systemd-boot.hook << EOF
[Trigger]
Type = Package
Operation = Upgrade
Target = systemd

[Action]
Description = Updating systemd-boot
When = PostTransaction
Exec = /usr/bin/bootctl update
EOF
    # systemd-boot requires this before running bootctl
    systemd-machine-id-setup --root="$MNT" >/dev/null 2>&1
    return 0
}

prep_for_syslinux() {
    local cfgdir="$MNT/boot/syslinux"
    local cfgsrcdir="/usr/lib/syslinux/bios/"
    EDIT_FILES[9]="/boot/syslinux/syslinux.cfg"

    if [[ $SYS == 'UEFI' ]]; then
        EDIT_FILES[9]="/boot/EFI/syslinux/syslinux.cfg"
        BOOT_CMDS[syslinux]="efibootmgr -c -d $BOOT_DEVICE -p $BOOT_PART_NUM -l /EFI/syslinux/syslinux.efi -L $DIST"
        cfgdir="$MNT/boot/EFI/syslinux"
        cfgsrcdir="/usr/lib/syslinux/efi64/"
    fi

    mkdir -p $cfgdir
    cp -r $cfgsrcdir $cfgdir/
    cat > $cfgdir/syslinux.cfg << EOF
UI menu.c32
PROMPT 0

MENU TITLE $DIST Syslinux Boot Menu
TIMEOUT 50
DEFAULT $DIST

LABEL $DIST
MENU LABEL $DIST Linux
LINUX ../vmlinuz-$KERNEL
APPEND root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV ")rw
INITRD ../initramfs-$KERNEL.img
$([[ $(grep 'GenuineIntel' /proc/cpuinfo) && -e $MNT/boot/intel-ucode.img ]] && echo -en "\ninitrd  /intel-ucode.img")

LABEL ${DIST}fallback
MENU LABEL $DIST Linux Fallback
LINUX ../vmlinuz-$KERNEL
APPEND root=$ROOT_PART_ID $([[ $LUKS_DEV ]] && echo -n "$LUKS_DEV ")rw
INITRD ../initramfs-$KERNEL-fallback.img
$([[ $(grep 'GenuineIntel' /proc/cpuinfo) && -e $MNT/boot/intel-ucode.img ]] && echo -en "\ninitrd  /intel-ucode.img")
EOF
    return 0
}

install_bootloader() {
    chroot_cmd "export PATH=/usr/local/sbin:/usr/local/bin:/usr/bin:/bin:/usr/bin/core_perl"

    local msg="$_InstBootloader $BOOTLOADER\n"
    [[ $BOOT_PART != "" ]] && msg="$msg\n$_InstBootDev $BOOT_PART\n"
    echo -e "$msg\nMountpoint: ${BOOT_MNTS[$SYS-$BOOTLOADER]}\n"

    # if not on an LVM we can use the UUID for booting
    if ! [[ $ROOT_PART =~ /dev/mapper ]]; then
        ROOT_PART_ID="UUID=$(blkid -s PARTUUID $ROOT_PART | sed 's/.*=//g; s/"//g')"
        [[ $BOOTLOADER == 'systemd-boot' ]] && ROOT_PART_ID="PART$ROOT_PART_ID"
    else
        # for LVM use the partition name eg. /dev/mapper/cryptroot
        ROOT_PART_ID="$ROOT_PART"
    fi

    # needed for os-prober module to work properly in the chroot
    mkdir -p /run/udev && mount --bind /run/udev /run/udev >/dev/null 2>&1
    BOOT_CMDS[$BOOTLOADER]="mkdir -p /run/udev && mount --bind /run/udev /run/udev ; ${BOOT_CMDS[$BOOTLOADER]}"

    if [[ $SYS == 'UEFI' ]]; then
        # make sure efivarfs has been mounted
        mount -o remount,rw -t efivarfs efivarfs /sys/firmware/efi/efivars >/dev/null 2>&1
        BOOT_CMDS[$BOOTLOADER]="mount -o remount,rw -t efivarfs efivarfs /sys/firmware/efi/efivars ; ${BOOT_CMDS[$BOOTLOADER]}"

        # remove old boot entries
        local esp="$MNT/boot/efi/EFI/"
        [[ ! -d $MNT/boot/efi/EFI && -d $MNT/boot/EFI ]] && esp="$MNT/boot/EFI/"
        find $esp -maxdepth 1 -mindepth 1 \( -name '[aA][rR][cC][hH][lL]abs' -o -name '[Bb][oO][oO][tT]' \) -type d -exec rm -rf '{}' \; >/dev/null 2>&1
    fi

    # sets up the bootloader config(s) and ${BOOT_CMDS[$BOOTLOADER]}
    prep_for_$BOOTLOADER

    # run the bootloader command
    chroot_cmd "${BOOT_CMDS[$BOOTLOADER]}"
    check_for_errors "${BOOT_CMDS[$BOOTLOADER]}"

    if [[ $SYS == 'UEFI' && $BOOTLOADER =~ (grub|syslinux) ]]; then
        local boot_dir="$DIST"
        [[ $BOOTLOADER == 'syslinux' ]] && boot_dir="syslinux"

        # copy efi stub to generic catch all
        uefi_boot_fallback "$boot_dir"
    fi
    return 0
}
