#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

install_main() {
    # this assumes all needed variables/settings are setup as needed
    # no additional user confirmation are performed aside from what is needed

    # unpack the whole filesystem to install directory $MNT
    oneshot install_base

    genfstab -U $MNT > $MNT/etc/fstab 2>$ERR
    check_for_errors "genfstab -U $MNT > $MNT/etc/fstab"

    # touch up fstab if we used a swapfile
    if [[ -f $MNT/swapfile ]]; then
        sed -i "s~${MNT}~~" $MNT/etc/fstab 2>$ERR
        check_for_errors "sed -i s~${MNT}~~ $MNT/etc/fstab"
    fi

    # run package operations before bootloader and mkinitcpio
    # due to the possibility of choosing lts kernel earlier
    oneshot update_mirrorlist
    oneshot update_system
    oneshot install_packages
    [[ $LOGIN_TYPE == 'lightdm' ]] && oneshot setup_lightdm

    run_mkinitcpio || return 1
    install_bootloader || return 1

    oneshot set_hwclock
    oneshot create_user || return 1
    return 0
}

install_base() {
    # unpack the main system
    tput cnorm
    clear && echo -e "\nUnpacking base filesystem..\n\n"
    rsync -ah --info=progress2 /run/archiso/sfs/airootfs/ $MNT/

    # remove archiso init files
    find $MNT/usr/lib/initcpio -name 'archiso*' -type f -exec rm '{}' \;
    rm -rf $MNT/etc/{mkinitcpio-archiso.conf,sudoers.d/g_wheel,polkit-1/rules.d/49-nopasswd_global.rules}

    # cleanup system permissions
    sed -i 's/volatile/auto/g' $MNT/etc/systemd/journald.conf
    sed -i "s/# %wheel ALL=(ALL) ALL/%wheel ALL=(ALL) ALL/g" $MNT/etc/sudoers

    # for virtual machines remove configs for xorg, these cause mouse issues
    [[ $VM ]] && rm -rf $MNT/etc/X11/xorg.conf.d

    # if not installing the lts kernel, copy the kernel image
    [[ $KERNEL != 'linux-lts' ]] && cp -f /run/archiso/bootmnt/arch/boot/x86_64/vmlinuz $MNT/boot/vmlinuz-linux

    setup_configs
    return 0
}

setup_configs() {
    # copy network settings
    cp -rf /etc/NetworkManager/system-connections $MNT/etc/NetworkManager/
    cp -f /etc/resolv.conf $MNT/etc/

    # set the locale and timezone
    sed -i "s/#en_US.UTF-8/en_US.UTF-8/g; s/#${LOCALE}/${LOCALE}/g" $MNT/etc/locale.gen
    sed -i "s/en_US.UTF-8/${LOCALE}/g" $MNT/etc/locale.conf
    cp -f $MNT/etc/locale.conf $MNT/etc/default/locale
    chroot_cmd "locale-gen" 2>$ERR
    check_for_errors 'locale-gen'
    chroot_cmd "ln -sf /usr/share/zoneinfo/$ZONE/$SUBZONE /etc/localtime" 2>$ERR
    check_for_errors "ln -sf /usr/share/zoneinfo/$ZONE/$SUBZONE /etc/localtime"

    # setup xorg vsync config for intel graphics
    if [[ $(lspci | grep ' VGA ' | grep 'Intel') != "" ]]; then
        cat > $MNT/etc/X11/xorg.conf.d/20-intel.conf <<EOF
Section "Device"
    Identifier  "Intel Graphics"
    Driver      "intel"
    Option      "TearFree" "true"
EndSection
EOF
    fi

    # set the keymaps
    cat > $MNT/etc/X11/xorg.conf.d/00-keyboard.conf <<EOF
# Use localectl(1) to instruct systemd-localed to update it.
Section "InputClass"
    Identifier      "system-keyboard"
    MatchIsKeyboard "on"
    Option          "XkbLayout" "$KEYMAP"
EndSection
EOF
    cat > $MNT/etc/default/keyboard <<EOF
# KEYBOARD CONFIGURATION FILE
# Consult the keyboard(5) manual page.
XKBMODEL=""
XKBLAYOUT="$KEYMAP"
XKBVARIANT=""
XKBOPTIONS=""
BACKSPACE="guess"
EOF
    # console keymap
    cat > $MNT/etc/vconsole.conf <<EOF
KEYMAP=$CONSOLE_MAP
FONT=$FONT
EOF

    # set the hostname
    echo "$HOSTNAME" > $MNT/etc/hostname
    cat > $MNT/etc/hosts << EOF
127.0.0.1   localhost
127.0.1.1   $HOSTNAME
::1     localhost ip6-localhost ip6-loopback
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
EOF
    return 0
}

create_user() {
    echo -e "\nSetting root password\n"
    chroot_cmd "echo 'root:$ROOT_PASS' | chpasswd"

    echo -e "$_UserSetBody"
    # edit the required files in /etc/ to swap the liveuser account name
    sed -i "s/${LIVE}/${NEWUSER}/g" $MNT/etc/{group,gshadow,passwd,shadow}

    # set standard groups for the new user
    local groups="rfkill,wheel,network,lp,storage,power,video,audio,lp,autologin"

    if [[ $AUTOLOGIN == true && $LOGIN_TYPE == 'lightdm' ]]; then
        groups="$groups,nopasswdlogin"
    elif [[ $AUTOLOGIN == true ]]; then
        sed -i "s/${LIVE}/${NEWUSER}/g" $MNT/etc/systemd/system/getty@tty1.service.d/autologin.conf
    fi

    chroot_cmd "mv -f /home/$LIVE /home/$NEWUSER"
    chroot_cmd "usermod -aG $groups $NEWUSER"
    chroot_cmd "echo '$NEWUSER:$USER_PASS' | chpasswd"
    chroot_cmd "chown -Rf $NEWUSER:users /home/$NEWUSER"

    setup_user_home
    return 0
}

setup_user_home() {
    local user_home="$MNT/home/$NEWUSER"

    sed -i "s/${LIVE}/${NEWUSER}/g" $user_home/.config/gtk-3.0/bookmarks \
        $user_home/.mozilla/firefox/{archlabs.default/prefs.js,archlabs.default/sessionstore.js}

    if [[ $AUTOLOGIN == true ]]; then
        if [[ $LOGIN_TYPE == 'lightdm' ]]; then
            rm -rf $user_home/.{zprofile,xinitrc}
        else
            sed -i "s/:-openbox/:-${LOGIN_WM}/g" $user_home/.xinitrc
            sed -i '/archlabs-installer/d' $user_home/.zprofile
            echo '[[ -z $DISPLAY && $XDG_VTNR -eq 1 ]] && exec startx -- vt1 &>/dev/null' >> $user_home/.zprofile
        fi
    else
        sed -i '/archlabs-installer/d' $user_home/.zprofile
        echo '[[ -z $DISPLAY && $XDG_VTNR -eq 1 ]] && exec startx -- vt1 &>/dev/null' >> $user_home/.zprofile
        sed -i "s/:-openbox/:-${LOGIN_WM}/g" $user_home/.xinitrc
    fi

    if ! [[ $INSTALL_WMS =~ openbox ]]; then
        rm -rf $user_home/.config/{openbox,ob-autostart,obmenu-generator}
    elif ! [[ $INSTALL_WMS =~ bspwm ]]; then
        rm -rf $user_home/.config/{bspwm,sxhkd}
    elif ! [[ $INSTALL_WMS =~ i3-gaps ]]; then
        rm -rf $user_home/.config/i3
    fi

    return 0
}

set_hwclock() {
    chroot_cmd "hwclock --systohc --utc"
    [[ $? != 0 ]] && chroot_cmd "hwclock --systohc --utc --directisa"
    return 0
}

update_mirrorlist() {
    echo -e "\nSorting the mirrorlist..\n"
    $MIRROR_CMD --verbose --save $MNT/etc/pacman.d/mirrorlist && return 0
    echo -e "\nAn error occurred while updating the mirrorlist.\n\nFalling back to automatic sorting...\n"
    reflector --score 100 -l 50 -f 10 --sort rate --verbose --save $MNT/etc/pacman.d/mirrorlist
    return 0
}

run_mkinitcpio() {
    local conf="$MNT/etc/mkinitcpio.conf"
    local add

    # setup a keyfile for LUKS.. Only when choosing grub and system is UEFI
    if [[ $LUKS -eq 1 && $SYS == 'UEFI' && $BOOTLOADER == 'grub' && $LUKS_PASS && $LUKS_UUID ]]; then
        luks_keyfile || return 1
    fi

    # new HOOKS needed in /etc/mkinitcpio.conf if we used LUKS and/or LVM
    (( LVM == 1 )) && add="lvm2"
    (( LUKS == 1 )) && add="encrypt$([[ $add != "" ]] && echo -n " $add")"
    sed -i "s/block filesystems/block ${add} filesystems ${MKINIT_HOOKS}/g" $conf

    tput civis
    chroot_cmd "mkinitcpio -p $KERNEL" 2>$ERR
    check_for_errors "mkinitcpio -p $KERNEL"

    return 0
}
